import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;  
/**
 * This is the code for a tile used in the 2048 game.
 * The tile is an image with a number printed on it and a background color
 * To create gaps between tiles, the tile is implemented like a framed picture.
 * The frame will cause the tiles to appear with a gap between them
 * 
 * @author M. Kadri 
 * @version 2 August 2014
 */
public class Tile extends Actor
{
    // Imagine the tile like a framed picture with some text written "0", "2", ..., "2048".
    final int FONT_SIZE = 40; // change as desired  
    final int TILE_SIZE = 90; // this is the size of the picture. Will have a color and a number printed on it
    final int BG_SIZE = 100;  // This is the size of the outer frame. Will be transparent

    private int value;  //0, 2, ..., 2048
    
    /**
     * Constructor
     */ 
    public Tile(int value){
            this.value=value;
            drawImage();
    }
    
    /**
     * Displayes the tile to the screen
     */
    public void drawImage(){
        Color fontColor = Color.BLACK; // change as desired    
        Color txtBgColor = new Color(0, 0, 0, 0); // transparent background for text

        // Create the tile image without the text
        GreenfootImage img = new GreenfootImage(TILE_SIZE, TILE_SIZE);  
        Color hexColor = new Color(getHexValue());
        img.setColor(hexColor); 
        img.fill();
        
        // no text on tile if its value is zero
        String text = "";
        if (this.value != 0)
            text += this.value;
            
        // the text image to be drawn on top of the tile image
        GreenfootImage txtImg = new GreenfootImage(text, FONT_SIZE, fontColor, txtBgColor); 
        
        //place the text image in the center of the tile image
        img.drawImage(txtImg, TILE_SIZE/2-txtImg.getWidth()/2, TILE_SIZE/2-txtImg.getHeight()/2);  
        setImage(img);  
    }
    
    /**
     * Returns the color of the tile based on the tile value
     */
    public int getHexValue(){
        int hexVal;
        String hexStr;
        if (this.value == 2)
            hexStr="e9e9e9";
        else if (this.value == 4)
            hexStr="e6d4ab";
        else if (this.value == 8)
            hexStr="f79d3d";
        else if (this.value == 16)
            hexStr="f28007";
        else if (this.value == 32)
            hexStr="f55e3b";
        else if (this.value == 64)
            hexStr="ff1212";
        else if (this.value == 128)
            hexStr="e9de84";    
        else if (this.value == 256)
            hexStr="f6e873";
        else if (this.value == 512)
            hexStr="f5e455";
        else if (this.value == 1024)
            hexStr="f7e12c";
        else if (this.value == 2048)
            hexStr="ffe400";
        else
            hexStr="e9e9e9";
        hexVal=Integer.parseInt(hexStr, 16);
        return hexVal;
    }

    // There is no act() method 
}
