import greenfoot.*; 
import java.awt.Color;
import java.util.List;
import java.util.Random;
/**
 * The 2048 game.
 * 
 * @author M. Kadri 
 * @version 2 August 2014
 */
public class World_2048 extends World
{
    final int GRID_SIZE = 4;
    int [][] grid = new int[GRID_SIZE][GRID_SIZE];
    int [][] savedGrid = new int[GRID_SIZE][GRID_SIZE];

    /**
     * Constructor 
     */
    public World_2048()
    {    
        // Create a new world with 400x400 cells with a cell size of 100 pixels.
        super(400, 400, 100); 
        populate();
    }

    /**
     * Create and display the initial grid with two cells showing a 2
     */
    public void populate(){
        newGrid();
        paintGrid();
    }
    
    /**
     * Create the initial grid with two cells showing a 2
     */
    public void newGrid(){
        for (int row = 0; row < GRID_SIZE; row ++){
            for (int col = 0; col < GRID_SIZE; col ++){
                grid[row][col]=0;
            }
        }
        //pick two random tiles and set their value to 2
        newTile(2);
        newTile(2);

    }
    
    /**
     * Create a new tile with the specified value at a random empty cell
     */
    public void newTile(int value){
        int [] empty_element;
        empty_element = findEmptyCell();
        if (empty_element == null)
            return;
        int x = empty_element[0];
        int y = empty_element[1];
        grid[x][y]= value;
    }

    /**
     * Randomly selects an empty grid cell and returns its location coordinates x and y
     */
    public int [] findEmptyCell(){
        // create two parallel arrays one to hold the x coordinate and one for the y coordinate
        // of 0 grid (empty)d cells
        int [] row_array = new int[GRID_SIZE*GRID_SIZE];
        int [] col_array = new int[GRID_SIZE*GRID_SIZE];
        int empty_elements=0;
        for (int row=0; row< GRID_SIZE; row++){
            for (int col=0; col<GRID_SIZE; col++){
                if (grid[row][col] == 0){
                    row_array[empty_elements]=row;
                    col_array[empty_elements]=col;
                    empty_elements ++;    
                }
            }
        }
        if (empty_elements==0)
            return null;
        Random randomGenerator = new Random();
        // pull a random number less than the number of 0 elements
        int randomi = randomGenerator.nextInt(empty_elements);
        // a two dimensional array to hold the row and col of the selected random cell
        int [] empty_element = new int[2];
        empty_element[0]=row_array[randomi];
        empty_element[1]=col_array[randomi];
        // return {row,col} of the selected random grid element
        return empty_element;
    }
   
    /**
     * Print the grid for initial testing and debugging
     */
    public void printGrid(){
        System.out.println();
        for (int row = 0; row < GRID_SIZE; row ++){
            for (int col = 0; col < GRID_SIZE; col ++){
                System.out.print("|"+grid[row][col]);
            }
            System.out.println("|");
        }
    }

    /**
     * Place the tiles on the screen
     */
    public void paintGrid(){
        for (int row = 0; row < GRID_SIZE; row ++){
            for (int col = 0; col < GRID_SIZE; col ++){
                //Tile tile = new Tile(grid[row][col],row,col);
                Tile tile = new Tile(grid[row][col]);
                addObject(tile,col,row);
            }
        }
    }

    /**
     * The act() method
     */
    public void act(){

        saveGrid();
        String key = Greenfoot.getKey();
        if (key != null){

            if (key.equals("left"))
            {
                moveLeft();
            }
            else if (key.equals("right"))
            {
                moveRight();
            }
            else if (key.equals("up"))
            {
                moveUp();
            }
            else if (key.equals("down"))
            {
                moveDown();
            }
            if (key.equals("left")||key.equals("right")||key.equals("up")||key.equals("down")){       
                if (gridChanged()){
                    newTile(2);
                }
                printGrid();
                clearWorld();
                paintGrid();

            }
        }

    }
    
    /**
     * This method is called when the up arrow key is presed
     */
    public void moveUp(){
        // Start with the grid's top row
        for (int c=0; c< GRID_SIZE; c++){
            // For each element
            // create an array to hold all elements of the column (vertical)
            int [] column= new int[GRID_SIZE];
            for (int r=0; r< GRID_SIZE; r++){
                column[r]=grid[r][c];
            }
            // Merge them
            column=merge(column);
            // Copy back into the original grid
            for (int r=0; r<GRID_SIZE; r ++)
                grid[r][c] = column[r];
        }
    }


    /**
     * This method is called when the down arrow key is presed
     */
    public void moveDown(){
        // Same as the mergeUpWard() method but we invert the array that holds
        // allelements of the column (vertcal) since we want to merge them downward
        // before we merge the array
        for (int c=0; c< GRID_SIZE; c++){
            // For each element
            // create an array to hold all elements of the column (vertical)
            int [] column= new int[GRID_SIZE];
            for (int r=0; r< GRID_SIZE; r++){
                column[r]=grid[r][c];
            }
            // Invert it so that the tail becomes the head
            column=invert(column);
            // Merge them
            column=merge(column);
            // invert it back to its original orientation before we copy it back to the grid
            column=invert(column);

            // Copy back into the original grid
            for (int r=0; r<GRID_SIZE; r ++)
                grid[r][c] = column[r];
        }
    }

    /**
     * This method is called when the left arrow key is presed
     */
    public void moveLeft(){
        for (int x=0; x< GRID_SIZE; x++){
            //System.out.println("\nrow:"+x);
            // array to contain all elements of a grid row (horizontal)
            int [] row= new int[GRID_SIZE];
            for (int y=0; y< GRID_SIZE; y++){
                row[y]=grid[x][y];
                //System.out.print(grid[x][y]);
            }
            //System.out.println();
            //printArray(row);
            row = stripZeroes(row);
            //System.out.println("stripped:");
            //printArray(row);
            row=merge(row);
            //System.out.println("merged:");
            //printArray(row);

            // copy stripped array back into the original grid
            for (int y=0; y<GRID_SIZE; y ++)
                grid[x][y] = row[y];
        }

    }

    /**
     * This method is called when the right arrow key is presed
     */
    public void moveRight(){
        for (int x=0; x< GRID_SIZE; x++){
            //System.out.println("\nrow:"+x);
            // array to contain all elements of a grid row (horizontal)
            int [] row= new int[GRID_SIZE];
            for (int y=0; y< GRID_SIZE; y++){
                row[y]=grid[x][y];
                //System.out.print(grid[x][y]);
            }
            //System.out.println();
            //printArray(row);
            row=invert(row);
           // System.out.println("inverted:");
           // printArray(row);
            row = stripZeroes(row);
           // System.out.println("stripped:");
           // printArray(row);
            row=merge(row);
           // System.out.println("merged:");
           // printArray(row);
            row=invert(row);

            // copy stripped array back into the original grid
            for (int y=0; y<GRID_SIZE; y ++)
                grid[x][y] = row[y];
        }

    }
    
    /**
     * Invert the row or column 4 element array array
     */
    public int[] invert(int[] array){
        int []inverted_array = new int[GRID_SIZE];
        int idx=GRID_SIZE-1;
        for (int element : array){
            inverted_array[idx]=element;
            idx --;
        }
        return inverted_array;
    }

    /**
    * Input is int array has GRID_SIZE elements 
    * |0|2|0|2|0|
    * This method moves all non zero elements to the left
    * |2|2|0|0|0|
    */
    public int[] stripZeroes(int [] row){
        
        int [] newRow = new int [row.length];
        int idx=0;
        for (int num : row){
            if (num != 0){
                newRow[idx]=num;
                idx ++;
            }
        }
        return newRow;

    }
    
    /**
     * This method moves non 0 array elements to the left
     */
    public int[] merge(int [] array){
    
        // Input is an int array
        // |2|0|0|2|0|
        // First we need to move all non zero numbers left
        // |2|0|0|2|0| --> |2|2|0|0|0|
        array = stripZeroes(array);
        //boolean merged = false;
        
        //Any two of a kind are added to form a single element closer to the left
        //|2|2|0|0|0| --> |4|0|0|0|0|
        //Note:
        //|2|2|2|2|0| --> |4|4|0|0|0| not |8|0|0|0|0|
        //|2|2|8|2|2| --> |4|8|4|0|0|
        for (int i=0; i<GRID_SIZE-1; i++){
            if ((array[i]== array[i+1])&&(array[i]!=0)){
                array[i] = array[i]*2;
                array[i+1] = 0;
                //merged = true;
            }
        } 
        // Lastly we need to move all non zero numbers left
        //|2|2|0|2|2| --> |4|0|0|4|0| 
        //            --> |4|4|0|0|0|
        array = stripZeroes(array);
        return array;
    }

    /**
     * Usefull when testing and debugging
     */
    public void printArray(int [] array){
        for (int num : array)
            System.out.print(num+" ");
        System.out.print("\n");
    }

    /**
     * Erace everything
     */
    public void clearWorld(){
        removeObjects(getObjects(null));
    }
    
    /**
     * Save the grid to help us determine if the new grid had changed.
     */
    public void saveGrid(){
        for (int row = 0; row<GRID_SIZE; row++){
            for (int col=0; col<GRID_SIZE; col++){
                savedGrid[row][col] = grid[row][col];
            }
        }
    }
    
    /**
     * Did the grid change or not
     */
    public boolean gridChanged(){
        for (int row = 0; row<GRID_SIZE; row++){
            for (int col=0; col<GRID_SIZE; col++){
                if (savedGrid[row][col] != grid[row][col]){
                    return true;
                }
            }
        }
        return false;
    }

}
